/* sshdos.c       Copyright (c) 2000 Nagy Daniel

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU Library General Public
 License along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <stdarg.h> 
#include "tcp.h"
#include "ssh.h"
#include "des.h"
#include "blowfish.h"
#include "version.h"


extern void ssh_gotdata(unsigned char *data);
extern void s_wrpkt_start(int type, int len);
extern void s_wrpkt(void);
extern int connect(void);
extern int ConChk(void);	 /* Check the keyboard for keystrokes */
extern int DoKey(void);		 /* Transmit a key sequence */
extern void ConOut(unsigned char);
extern void KeyInit(void);
extern void VidInit(void);
extern void VTInit(void);

struct Packet pktin = { 0, 0, 0, NULL, 0 };
struct Packet pktout = { 0, 0, 0, NULL, 0 };

int cipher=0;				/* cipher of or off*/
int cipher_type=SSH_CIPHER_3DES;	/* type of cipher */

unsigned char inbuf[532];  /* buffer for incoming raw packet */

tcp_Socket s;		/* socket */
int status;		/* status of communication */
unsigned char *username=NULL;
char *remotehost;
char *term="ansi"; /* terminal type */
int ssh_port=22;


/* fatal error handler */
void fatal(const char *fmt, ...)
{
va_list ap;
char buf[1024];

  va_start(ap, fmt);
  vsprintf(buf, fmt, ap);
  va_end(ap);
  fprintf(stderr, "%s\n", buf);
  sock_close(&s);
  exit(255);
}

/* Client loop. This runs until SSH connection is terminated */
int dosession(void)
{
int len,i;

        while (ConChk())
		DoKey();

	sock_tick(&s, &status); /* TCP wait */
	len=sock_fastread(&s,inbuf,4); /* Read some data */
	if(!len) return(0); /* Begin loop again if none */
	for (i = len = 0; i < 4; i++) len = (len << 8) + inbuf[i]; /* Get packet size */
	sock_read(&s,inbuf+4,len + 8 - (len%8)); /* Read it */
	ssh_gotdata(inbuf); /* uncrypt and get valuable data */

	if(pktin.type==SSH_SMSG_STDOUT_DATA || pktin.type==SSH_SMSG_STDERR_DATA)
		{

		for (i = 0; i < pktin.length; i++)
			ConOut(pktin.body[i+4]);

		return(0);
		}
	else if(pktin.type==SSH_SMSG_EXITSTATUS)
		{
		s_wrpkt_start(SSH_CMSG_EXIT_CONFIRMATION,0); /* Send exit if requested */
		s_wrpkt();
                return(EXIT_SSH);
		}
	else if(pktin.type==SSH_MSG_DISCONNECT)
		{
		pktin.body[pktin.length+4]=0; /* Connection reset by other party */
		puts(pktin.body+4);
		return(EXIT_SSH);
		}
	else if(pktin.type==SSH_SMSG_SUCCESS)
		{
		return(0);
		}
	else
		{
		printf("Unsupported packet received. Type: %d\n",pktin.type);
		return(0);
		}

sock_err:
  switch (status)
  {
    case 1 : cputs ("Connection closed");
             break;
    case -1: cputs ("REMOTE HOST CLOSED CONNECTION");
             break;
  }
  return(EXIT_SSH);

}


/* Get command line arguments */

void getargs(int argc, char *argv[])
{
int i;
char *s;
char *usage="Usage: sshdos [-c cipher] [-t term_type] [-P port] username remotehost\n"
	    "Valid ciphers are: 3des, blowfish. Default is 3des\n"
	    "Valid terminal types are: ansi, vt100. Default is ansi";

    for (i = 1; i < argc; ++i)
    {
	s = argv[i];
	if (*s != '-') break;
	switch (*++s)
	{
	case '\0':
	    fatal(usage);
	    return;

	case 'c':
	    if (*++s)
		{
		 if(!strcmp(s,"3des")) cipher_type = SSH_CIPHER_3DES;
		 else if(!strcmp(s,"blowfish")) cipher_type = SSH_CIPHER_BLOWFISH;
		 else fatal(usage);
		}
	    else if (++i < argc)
		{
		if(!strcmp(argv[i],"3des")) cipher_type = SSH_CIPHER_3DES;
		else if(!strcmp(argv[i],"blowfish")) cipher_type = SSH_CIPHER_BLOWFISH;
		else fatal(usage);
		}
	    else
		fatal(usage);
	    continue;

	case 't':
	    if (*++s)
		{
		 if(!strcmp(s,"ansi") || !strcmp(s,"vt100")) term = s;
		 else fatal(usage);
		}
	    else if (++i < argc)
		{
		 if(!strcmp(argv[i],"ansi") || !strcmp(argv[i],"vt100")) term = argv[i];
		 else fatal(usage);
		}
	    else
		fatal(usage);
	    continue;

	case 'P':
	    if (*++s)
		 ssh_port = atoi(s);
	    else if (++i < argc)
		ssh_port = atoi(argv[i]);
	    else
		fatal(usage);
	    continue;

	default:
	    fatal(usage);
	} /* end switch */

    }

no_more_options:;
    if (i + 2 != argc) fatal(usage);
    username = argv[i];
    remotehost = argv[i+1];
}



/* main program */
int main(int argc, char **argv)
{
int i, ch;

	printf("SSHDOS v%s\n", SSH_VERSION);

getargs(argc, argv);

/* Initalizations */

    srand (time(NULL)); /* Initialize random number generator */

    if(connect()) return(1); /* Connect to remote server */

    KeyInit(); /* Initialize VT100 module */
    VidInit();
    VTInit();

/* Loop until session end */
   while(EXIT_SSH!=dosession());

/* Close TCP socket */
sock_close(&s);
sock_wait_closed(&s, sock_delay, NULL, &status);
return(0);

sock_err:
  switch (status)
  {
    case 1 : cputs ("Connection closed");
             break;
    case -1: cputs ("REMOTE HOST CLOSED CONNECTION");
             break;
  }
  return(1);
}

